#include <iostream>
#include "CGAL/Exact_predicates_inexact_constructions_kernel.h"
#include "CGAL/Triangulation_vertex_base_with_info_3.h"
#include "CGAL/Triangulation_3.h"
#include "CGAL/Triangulation_hierarchy_3.h"
#include "CGAL/Delaunay_triangulation_3.h"
#include "genbench.h"
#include "main.h"

using namespace std;

// From CGAL user manual...
typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Triangulation_vertex_base_with_info_3<int, K>	Vb;
typedef CGAL::Triangulation_hierarchy_vertex_base_3<Vb>		Vbh;
typedef CGAL::Triangulation_data_structure_3<Vbh>		Tds;
typedef CGAL::Delaunay_triangulation_3<K,Tds>			Dt;
typedef CGAL::Triangulation_hierarchy_3<Dt>			Dh;


int main(int argc, char **argv)
{
	Dh dt;
	Dh::Vertex_handle *handles;
	int frame, i;
	double x, y, z;
	
	gb_init((char *)"Delaunay Triangulation with CGAL", argc, argv);
	
	handles = new Dh::Vertex_handle[gb_nobj()];

	cout << "Generating " << gb_nobj() << " objects..." << endl;
	
	for (i = 0; i < gb_nobj(); i++) {
		gb_obj_coord(i, &x, &y, &z);

		Dh::Point p(x, y, z);

		handles[i] = dt.insert(p);
		handles[i]->info() = i;
	}
	
	for (frame = 0; 1; frame++) {
		gb_timer_start(GB_COLLISION_PHASE);
		gb_timer_start(GB_POSITION_UPDATE);

		for (i = 0; i < gb_nobj(); i++) {
			gb_obj_coord(i, &x, &y, &z);
			Dh::Point p(x, y, z);

			dt.move_point(handles[i], p);
		}

		gb_timer_stop(GB_POSITION_UPDATE);
		
		Dh::Finite_edges_iterator ei;

		for (ei = dt.finite_edges_begin();
		ei != dt.finite_edges_end(); ++ei) {
			Dh::Edge e = *ei;
			Dh::Cell_handle c = e.first;
			Dh::Vertex_handle v1 = c->vertex(e.second);
			Dh::Vertex_handle v2 = c->vertex(e.third);

			gb_collision(v1->info(), v2->info());
		}

		gb_timer_stop(GB_COLLISION_PHASE);
		gb_next_step();
	}
	return(0);
}


