#include <iostream>
#include "genbench.h"
#include "NBody.H"

using namespace std;

static void zero_trans(double trans[][4])
{
	int i, j;
	
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 4; j++) {
			trans[i][j] = 0;
		}
	}
}

static void set_trans(double trans[][4], double x, double y, double z)
{
	zero_trans(trans);
	trans[0][3] = x;
	trans[1][3] = y;
	trans[2][3] = z;
}

int main(int argc, char **argv)
{
	RAPID_model a;
	Elem *e;
	NBody system;
	double trans[3][4];
	int i;
	double x, y, z;
	
	gb_init((char *)"Sweep & Prune from a modified V-Collide", argc, argv);

	cout << "Generating objects (N = " << gb_nobj()
		<< ") This may be slow for N > 5000..." << endl;
	
	for (i = 0; i < gb_nobj(); i++) {
		a.center[0] = a.center[1] = a.center[2] = 0.0;
		a.radius = gb_obj_radius(i);
		system.AddObject(i, &a);

		gb_obj_coord(i, &x, &y, &z);
		set_trans(trans, x, y, z);
		system.UpdateTrans(i, trans);
	}
	
	while(1) {
		gb_timer_start(GB_COLLISION_PHASE);
		gb_timer_start(GB_POSITION_UPDATE);

		for (i = 0; i < gb_nobj(); i++) {
			gb_obj_coord(i, &x, &y, &z);
			set_trans(trans, x, y, z);
			system.UpdateTrans(i, trans);
		}
		
		gb_timer_stop(GB_POSITION_UPDATE);
		
		for (i = 0; i < system.overlapping_pairs.size; i++) {
			e = system.overlapping_pairs.arr[i];
			while (e) {
				gb_collision(i, e->id);
				e = e->next;
			}
		}

		gb_timer_stop(GB_COLLISION_PHASE);
		gb_next_step();
	}
	return(0);
}


